// Define the dimensions for the visualization
const margin = {
    top: 50,
    right: 130,
    bottom: 40,
    left: 150,
  },
  width = 1700 - margin.left - margin.right,
  height = 1700 - margin.top - margin.bottom;

// Append the svg object to the div container
const svg = d3
  .select('#tree-container')
  .append('svg')
  .attr('width', width + margin.right + margin.left)
  .attr('height', height + margin.bottom + margin.top)
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',' + margin.top + ')',
  );

// Create a tree layout
const treemap = d3.tree().size([height, width]);

// Load the external data
d3.csv(
  'https://gist.githubusercontent.com/VarshaYarragunta/31e86c416d93cb1c43b83159b1c486d1/raw/56f958ec71815755f449ade3c1457ec2593043bc/amazon_prime_titles_more_creative_filled.csv',
).then(function (data) {
  // Process the data to fit the tree structure
  let rootData = { name: 'Root', children: [] };
  let types = {};
  data.forEach((d) => {
    if (!types[d.type]) {
      types[d.type] = { name: d.type, children: [] };
      rootData.children.push(types[d.type]);
    }
    let genre = d.listed_in.split(',')[0]; // Only consider the first genre
    let foundGenre = types[d.type].children.find(
      (g) => g.name === genre,
    );
    if (!foundGenre) {
      foundGenre = { name: genre, children: [] };
      types[d.type].children.push(foundGenre);
    }
    foundGenre.children.push({ name: d.title }); // Add title as leaf node
  });

  // Create the root hierarchy node
  let root = d3.hierarchy(rootData, (d) => d.children);

  // Assigns the x and y position for the nodes
  treemap(root);

  // Define colors for different depths
  const colorScale = d3.scaleOrdinal(d3.schemeCategory10);

  // Add links between nodes
  svg
    .selectAll('.link')
    .data(root.links())
    .enter()
    .append('path')
    .attr('class', 'link')
    .style('stroke', (d) => colorScale(d.target.depth))
    .attr(
      'd',
      d3
        .linkHorizontal()
        .x((d) => d.y)
        .y((d) => d.x),
    );

  // Add each node as a group
  const node = svg
    .selectAll('.node')
    .data(root.descendants())
    .enter()
    .append('g')
    .attr(
      'class',
      (d) =>
        'node' +
        (d.children ? ' node--internal' : ' node--leaf'),
    )
    .attr('transform', (d) => `translate(${d.y},${d.x})`);

  // Differentiate shapes for nodes
  node
    .append((d) =>
      d.children
        ? document.createElementNS(
            d3.namespaces.svg,
            'rect',
          )
        : document.createElementNS(
            d3.namespaces.svg,
            'circle',
          ),
    )
    .attr('width', 20) // For rect only
    .attr('height', 20) // For rect only
    .attr('r', 10) // For circle only
    .style('fill', (d) => colorScale(d.depth))
    .style('stroke', '#fff'); // Add white border for better visibility

  // Adjust position for rectangle to align it similar to circle
  node
    .filter((d) => d.children)
    .selectAll('rect')
    .attr('x', -10)
    .attr('y', -10);

  // Add the text to each node
  node
    .append('text')
    .attr('dy', '0.31em')
    .attr('x', (d) => (d.children ? -15 : 15))
    .attr('y', (d) => (d.children ? 0 : 5))
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => d.data.name)
    .style('fill', '#333') // Change text color for better readability
    .style('font-size', '12px'); // Adjust font size for better fit
});
